Documentation
¶
Index ¶
Constants ¶
const ( SetProgramTips = `You can type in a program or import a previously saved program.` ProgramAddrTips = `Your program will be written to memory starting from this address.` ProgramCounterStartAtTips = `The program will start running from this address, that is, this is the initial value of PC.` MaxStepsTips = `The program will stop after executing this number of instructions.` HaultAtTips = `The program will stop when it reaches this address, and the instructions at this address will not be executed.` MemorySettingsTips = `` /* 318-byte string literal not displayed */ )
const UserGuide = `` /* 578-byte string literal not displayed */
Variables ¶
This section is empty.
Functions ¶
Types ¶
type MemorySetting ¶
type ProgramConfig ¶
type ProgramConfig struct {
ProgramPath string `yaml:"-"`
ProgramInHexFormat []string `yaml:"programInHexFormat"`
ProgramAddr int `yaml:"programAddr"` // 程序从内存的这个地址开始写入
ProgramCounterStartAt int `yaml:"programCounterStartAt"` // 程序从内存的这个地址开始运行
MaxSteps int `yaml:"maxSteps"` // 程序运行指令个数的上限
HaltAt int `yaml:"haltAt"` // 程序运行到这个地址就会停止(该地址的指令不会被执行)
MemorySettings []MemorySetting `yaml:"memorySettings"` // 初始化内存数据
}
type SimulatorCli ¶
type SimulatorCli struct {
// contains filtered or unexported fields
}
var SimulatorCliSingleton *SimulatorCli
func (*SimulatorCli) ClearMainGrid ¶
func (s *SimulatorCli) ClearMainGrid()
清除当前组件,不清除会导无法正常focus。因为grid add item 时并不会清理同一个位置上的旧item,而是堆叠上去
func (*SimulatorCli) Run ¶
func (s *SimulatorCli) Run()
Click to show internal directories.
Click to hide internal directories.